/* -------------------------------------------------------------------- */
/* SMS Client, send messages to mobile phones and pagers		*/
/*									*/
/* proximus_web.c							*/
/*									*/
/*  Copyright (C) 2001 Koenraad Heijlen					*/ 
/*  Copyright (C) 1997,1998,1999 Angelo Masci				*/
/*  									*/
/*  This library is free software; you can redistribute it and/or	*/
/*  modify it under the terms of the GNU Library General Public		*/
/*  License as published by the Free Software Foundation; either	*/
/*  version 2 of the License, or (at your option) any later version.	*/
/*									*/
/*  This library is distributed in the hope that it will be useful,	*/
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/
/*  Library General Public License for more details.			*/
/*									*/
/*  You should have received a copy of the GNU Library General Public	*/
/*  License along with this library; if not, write to the Free		*/
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/
/*									*/
/*  You can contact the author at this e-mail address:			*/
/*									*/
/*  vipie@ulyssis.org							*/
/*									*/
/* -------------------------------------------------------------------- */
/* $Id: proximus_web.c,v 1.2 2002/03/26 14:09:24 vipie Exp $
   -------------------------------------------------------------------- */

/* From :Koenraad Heijlen 
 * -----------------------
 * (KH from now on)
 * The old advalvas code isn't any longer usable, this is a rewrite
 * wich uses nearly all of the oldcode but with some changes.
 * It now uses the official proximus page, aka Proximus Interactive.
 * ---------------------
 * Changes :
 *              * url
 *              * prefix parsing
 *              * the POST string
 *              * document return values *not done yet*
 *                they all give "enexpected" for now
 *              * again they changed it all ;) new url...
 *                                                                    */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "common/common.h"
#include "logfile/logfile.h"
#include "driver.h"
#include "error.h"
#include "comms/comms.h"
#include "comms/http.h"
#include "resource/resource.h"

/* -------------------------------------------------------------------- */

static struct proximus_web_env
{
	DRIVER_DEFAULT_ENV def;

	/* Place any extended driver	*/ 
	/* variables here 		*/

	char 	*url;

} driver_env;

/* -------------------------------------------------------------------- */

static 	RESOURCE resource_list[] = 
	{
		{ RESOURCE_STRING,  "SMS_comms_params", 	0, 1, NULL, 0,  "8N1",      0, 	  &(driver_env.def.comms_params)  	},
		{ RESOURCE_STRING,  "SMS_centre_number", 	0, 1, NULL, 0,  NULL,       0, 	  &(driver_env.def.centre_number)  	},
		{ RESOURCE_NUMERIC, "SMS_baud", 		0, 1, NULL, 0,  NULL,       9600, &(driver_env.def.baud)  		},
		{ RESOURCE_NUMERIC, "SMS_deliver_timeout", 	0, 0, NULL, 0,  NULL,       30,   &(driver_env.def.deliver_timeout)  	},
		{ RESOURCE_NUMERIC, "SMS_timeout", 		0, 0, NULL, 0,  NULL,       10,   &(driver_env.def.timeout)  		},
		{ RESOURCE_NUMERIC, "SMS_write_timeout", 	0, 0, NULL, 0,  NULL,       10,   &(driver_env.def.write_timeout)  	},
		{ RESOURCE_NUMERIC, "SMS_max_deliver", 		0, 0, NULL, 0,  NULL,       3,    &(driver_env.def.max_deliver)  	},
		{ RESOURCE_STRING,  "SMS_url", 			0, 1, NULL, 0,  "http://sms.proximus.be/WMS/ProxiText?smsaction=send&lan=en&laf=default", 	    0,    &(driver_env.url)			},
		{ RESOURCE_NULL,     NULL, 			0, 1, NULL, 0,  NULL,       0,    NULL  				}
	};

/* -------------------------------------------------------------------- */

#define DELIVERTIMEOUT 		(driver_env.def.deliver_timeout)
#define TIMEOUT 		(driver_env.def.timeout)
#define WRITETIMEOUT 		(driver_env.def.write_timeout)

/* -------------------------------------------------------------------- */

#define FD			(driver_env.def.fd)

/* -------------------------------------------------------------------- */

static int PROXIMUS_WEB_dial(DRIVER_DEFAULT_ENV *env);
static int PROXIMUS_WEB_sendmessage(char *msisdn, char *message);
static void PROXIMUS_WEB_hangup(DRIVER_DEFAULT_ENV *env);

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int PROXIMUS_WEB_dial(DRIVER_DEFAULT_ENV *env)
{
	return 0;
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */

struct intlprefix_struct {

	char 	*prprefix,
		*intlprefix;

} intlprefix_map[] = { 

        { "0472",  "32472"      },
        { "0473",  "32473"      },
        { "0474",  "32474"      },
        { "0475",  "32475"      },
        { "0475",  "3275"       },
        { "0476",  "32476"      },
        { "0477",  "32477"      },
        { "0478",  "32478"      },
        { "0479",  "32479"      },
	{  NULL,   NULL 	}
};

/* KH: the former 075 prefix now is the 0475 prefix
 * i think this will make the translation easy and even
 * unnoticed :) (not a GOOD THING (TM))
 * Also they have more prefixes 472 - 479
 */


static int PROXIMUS_WEB_sendmessage(char *msisdn, char *message)
{
	int	document_len,
		i;

	char	data[1024],
		selectedintlprefix[5],
		*document,
		*escaped_message,
		*prefix;

	char	*temp;

	/* ------------------------------------------------------------ */

	for (i=0; intlprefix_map[i].prprefix != NULL; i++)
	{
		if (strncmp(intlprefix_map[i].intlprefix, msisdn, strlen(intlprefix_map[i].intlprefix)) == 0)
		{
			break;
		} 
	}

	if (intlprefix_map[i].intlprefix == NULL)
	{

		fprintf(stderr, "Number '%s' not supported\n", msisdn);
		exit(1);
	}
	else
	{       
		prefix = intlprefix_map[i].prprefix;
		temp = msisdn + strlen(intlprefix_map[i].prprefix) + 1;
		strncpy(selectedintlprefix , temp, strlen(msisdn) - strlen(intlprefix_map[i].prprefix));
	}

	escaped_message = escape_input(message);
	if (escaped_message == NULL)
	{
		exit(-1);
	}

	/* ------------------------------------------------------------ */


	sms_snprintf(data, 1024, "prefix=%s&suffix=%s&smsmessage=%s",
		prefix,
		selectedintlprefix,
		escaped_message);

	free(escaped_message);


	/* ------------------------------------------------------------ */


	document = get_document(driver_env.url, &document_len, "POST", data, NULL);
	if (document != NULL)
	{
		lprintf(LOG_VERYVERBOSE, "%s\n", document);


		if (strstr(document, "Thank you for using SMS On Line. Your message was sent succesfully.") != NULL)
		{
			printf("Your message has been successfully queued for transmission\n");
		}
		else if (strstr(document, "The subscriber number you entered was not recognised by SMS On Line.") != NULL)
		{
			fprintf(stderr, "Error: Proximus does not accept the number you entered.\n");
			return -1;
		}
		else if (strstr(document, "Your message is too long.") != NULL)
		{
			fprintf(stderr, "Error: The Message you are trying to send is too long.\n");
			return -1;
		}
		else if (strstr(document, "A transmission error has occured between your browser and SMS On Line.") != NULL)
		{
			fprintf(stderr, "Error: A transmission error has occured... retry and submit to vipie@ulyssis.org please. ...\n");
			return -1;
		}
		else
		{	fprintf(stderr, "Error: Trying to Deliver message - Unexpected response\n");
			return -1;
		}

		free(document);
	}
	else
	{
		lprintf(LOG_ERROR, "Trying to Deliver message - Failed to connect\n");
		return -1;
	}

	return 0;
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void PROXIMUS_WEB_hangup(DRIVER_DEFAULT_ENV *env)
{	
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
DEVICE_ENTRY proximus_web_device = {

	"PROXIMUS_WEB",
	"0.3",
	resource_list,
	(DRIVER_DEFAULT_ENV *)(&driver_env),

	default_init,
	default_main,
	default_validate_numeric_id,
	PROXIMUS_WEB_dial,
	PROXIMUS_WEB_hangup,
	default_send_disconnect,
	default_single_deliver,
	PROXIMUS_WEB_sendmessage,
	default_login
};



